# -*- coding: utf-8 -*-
# module for scorethai data, loader.

import re, yaml

class Song(yaml.YAMLObject):
    """Yaml data type: Song."""
    yaml_tag = u'!song'
    yaml_flow_style = True
    
    # default attribute
    id = ''
    title = ''
    columns = 8
    body = []
    
    def __init__(self, id, title, columns, body):
        self.id = id
        self.title = title
        self.columns = columns
        self.body = body            # src of body before parse
    def __repr__(self):
        return '%s(id=%r, title=%r, columns=%r, body=%r)' % \
            (self.__class__.__name__, self.id, self.title, self.columns, self.body)
    def __str__(self):
        return '%s(\n id=%r\n title=%r\n columns=%r\n body=%r\n)\n' % \
            (self.__class__.__name__, self.id, self.title, self.columns, self.body)

    # regexp to parse song body
    reCell = re.compile(r'[,\t]\s*\n?|\s*\n')
    reNewline = re.compile(r'[\s,]*/')

    def parse(self):
        """Parse body of song, return list of Body."""
        bodies = []
        for src in self.body:
            if type(src) == dict: # lines that have keyword: columns, text, etc..
                for k, v in src.iteritems():
                    if k == 'columns':
                        t = Body.COLUMNS
                    elif k == 'text':
                        t = Body.TEXT
                    else:
                        msgf('* unknown keyword: %r in body: %r', k, src)
                        continue
                    bodies.append(Body(t, v))
            else:
                # normal cells, split by cell separator
                cells = [s.strip() for s in self.reCell.split(src)]
                
                # remove last empty cells
                if cells and not cells[-1]:
                    del cells[-1]
                    
                for s in cells:
                    for m in self.reNewline.finditer(s):
                        # found newline-mark, separate to the another cell
                        # eg. "a b / c" -> ["a b", newline, "c"]
                        s1 = s[:m.start()].strip()
                        if s1:
                            bodies.append(Body(Body.CELL, s1))
                        bodies.append(Body(Body.NEWLINE, ''))
                        s = s[m.end():].strip()
                        if not s:
                            break # no date after newline
                    else: # no newline, append whole as body
                        bodies.append(Body(Body.CELL, s))
        return bodies

class Body():
    """Parsed body.
    typ: data type.
    val: string value.
    """
    # types
    COLUMNS, TEXT, NEWLINE, CELL = 'c', ':', '/', ' '
    
    def __init__(self, typ, val):
        self.typ = typ
        self.val = val
    def __str__(self):
        return u'<' + str(self.typ) + u':' + repr(self.val) + u'>'
    __repr__ = __str__
