# -*- coding: utf-8 -*-
# module for tag formatter.
#

import sys, re

def msgf(s, *args):
    print >>sys.stderr, str(s) % args

class CellBuf(list):
    """Lines buffer for cells. self is list of line, line is list of cells.
    
    self ::= [ line,... ]
    line ::= [ cell-string,... ]
    cell-string ::= string
    """
    
    def __init__(self, maxcol, emptycell):
        self.setcol(maxcol)
        self.emptycell = emptycell
        self.cells = []
        
    def setcol(self, maxcol):
        self.maxcol = maxcol
        
    def flush(self, fill=True):
        """Add cells to self."""
        if not self.cells:
            return
        if fill:    # fill line by empty cells
            self.cells += [ self.emptycell ] * (self.maxcol - len(self.cells))
        # concat tagged cells and make a tagged line
        self.append(self.cells)
        self.cells = []
        
    def add_cell(self, s):
        """Add string as cell, flush line if filled columns."""
        self.cells.append(s)
        if len(self.cells) >= self.maxcol:
            self.flush(fill=True)
            
    def add_line(self, s):
        """Add string as a whole line."""
        self.flush(fill=True)
        self.cells.append(s)
        self.flush(fill=False)
        
class TagData(dict):
    """Format tags, use datadict to pick data.
    { key: [open-tag, close-tag],... }
    """
    datadict = {}
    
    def setdata(self, datadict):
        """Set data as datadict.
        datadict: dict to pick up data src.
        """
        self.datadict = datadict

    def update(self, v):
        """set format tag.
        v: value to set. must be dict.
        """
        msgf(' format keys: %s', ', '.join(v.keys()))
        if type(v) != dict:
            msgf('* format: value must be mapping: %r', v)
            sys.exit(1)
        dict.update(self, v)
        
    def tag_data(self, key, s=None):
        """Return string enclosed by format tag.
        key: format-key.
        s: string.
        """
        if key not in self:
            msgf('* format key: %r is not defined', key)
            sys.exit(1)
        lst = self[key]
        if type(lst) != list or len(lst) != 2:
            msgf('* format[%r] must has [open-tag, close-tag]', key)
            sys.exit(1)
        r = u''
        try:
            r += lst[0] % self.datadict
            if s:
                r += s
            r += lst[1] % self.datadict
        except KeyError, e:
            msgf('* format[%r]\n%r\n* data[%s] is not set.', key, lst, e)
            sys.exit(1)
        return r

if __name__=="__main__":
    class Format(TagData):
        def tag_table(self, lines):
            s = ''.join(
                [self.tag_data('line',
                    ''.join(
                        [self.tag_data('cell', cell)
                            for cell in line]))
                                for line in lines])
            return self.tag_data('table', s)
        def tag_page(self, s):
            return self.tag_data('page', s)
    tagdata = dict({
        'page' : [ '<page title="%(title)s>\n', '</page>\n' ],
        'table' : [ '<table>\n', '</table>\n' ],
        'line' : [ '<line>', '</line>\n' ],
        'cell' : [ '<cell>', '</cell>' ],
    })
    format = Format(tagdata)
    data = { 'title': 'this is a title' }
    format.setdata(data)
    lines = [['A', 'B', 'C'], ['D', 'E'], ['F']]
    s = format.tag_table(lines)
    print format.tag_page(s)
    # output:
    #<page title="this is a title>
    #<table>
    #<line><cell>A</cell><cell>B</cell><cell>C</cell></line>
    #<line><cell>D</cell><cell>E</cell></line>
    #<line><cell>F</cell></line>
    #</table>
    #</page>
